# encoding: utf8

import keyboard
from utils import *

code =  '''
def add_hotkey(hotkey, callback, args=(), suppress=False, timeout=1, trigger_on_release=False):
    """
    Invokes a callback every time a hotkey is pressed. The hotkey must
    be in the format `ctrl+shift+a, s`. This would trigger when the user holds
    ctrl, shift and "a" at once, releases, and then presses "s". To represent
    literal commas, pluses, and spaces, use their names ('comma', 'plus',
    'space').

    - `args` is an optional list of arguments to passed to the callback during
    each invocation.
    - `suppress` defines if successful triggers should block the keys from being
    sent to other programs.
    - `timeout` is the amount of seconds allowed to pass between key presses.
    - `trigger_on_release` if true, the callback is invoked on key release instead
    of key press.

    The event handler function is returned. To remove a hotkey call
    `remove_hotkey(hotkey)` or `remove_hotkey(handler)`.
    before the hotkey state is reset.

    Note: hotkeys are activated when the last key is *pressed*, not released.
    Note: the callback is executed in a separate thread, asynchronously. For an
    example of how to use a callback synchronously, see `wait`.

    Examples:

        # Different but equivalent ways to listen for a spacebar key press.
        add_hotkey(' ', print, args=['space was pressed'])
        add_hotkey('space', print, args=['space was pressed'])
        add_hotkey('Space', print, args=['space was pressed'])
        # Here 57 represents the keyboard code for spacebar; so you will be
        # pressing 'spacebar', not '57' to activate the print function.
        add_hotkey(57, print, args=['space was pressed'])

        add_hotkey('ctrl+q', quit)
        add_hotkey('ctrl+alt+enter, space', some_callback)
    """
    if args:
        callback = lambda callback=callback: callback(*args)

    _listener.start_if_necessary()

    steps = parse_hotkey_combinations(hotkey)

    event_type = KEY_UP if trigger_on_release else KEY_DOWN
    if len(steps) == 1:
        # Deciding when to allow a KEY_UP event is far harder than I thought,
        # and any mistake will make that key "sticky". Therefore just let all
        # KEY_UP events go through as long as that's not what we are listening
        # for.
        handler = lambda e: (event_type == KEY_DOWN and e.event_type == KEY_UP and e.scan_code in _logically_pressed_keys) or (event_type == e.event_type and callback())
        remove_step = _add_hotkey_step(handler, steps[0], suppress)
        def remove_():
            remove_step()
            del _hotkeys[hotkey]
            del _hotkeys[remove_]
            del _hotkeys[callback]
        # TODO: allow multiple callbacks for each hotkey without overwriting the
        # remover.
        _hotkeys[hotkey] = _hotkeys[remove_] = _hotkeys[callback] = remove_
        return remove_

    state = _State()
    state.remove_catch_misses = lambda: None
    state.remove_last_step = None
    state.suppressed_events = []
    state.last_update = float('-inf')
    
    def catch_misses(event, force_fail=False):
        if (
                event.event_type == event_type
                and state.index
                and event.scan_code not in allowed_keys_by_step[state.index]
            ) or (
                timeout
                and _time.monotonic() - state.last_update >= timeout
            ) or force_fail: # Weird formatting to ensure short-circuit.

            state.remove_last_step()

            for event in state.suppressed_events:
                if event.event_type == KEY_DOWN:
                    press(event.scan_code)
                else:
                    release(event.scan_code)
            del state.suppressed_events[:]

            index = 0
            set_index(0)
        return True

    def set_index(new_index):
        state.index = new_index

        if new_index == 0:
            # This is done for performance reasons, avoiding a global key hook
            # that is always on.
            state.remove_catch_misses() 
            state.remove_catch_misses = lambda: None
        elif new_index == 1:
            state.remove_catch_misses()
            # Must be `suppress=True` to ensure `send` has priority.
            state.remove_catch_misses = hook(catch_misses, suppress=True)

        if new_index == len(steps) - 1:
            def handler(event):
                if event.event_type == KEY_UP:
                    remove()
                    set_index(0)
                accept = event.event_type == event_type and callback() 
                if accept:
                    return catch_misses(event, force_fail=True)
                else:
                    state.suppressed_events[:] = [event]
                    return False
            remove = _add_hotkey_step(handler, steps[state.index], suppress)
        else:
            # Fix value of next_index.
            def handler(event, new_index=state.index+1):
                if event.event_type == KEY_UP:
                    remove()
                    set_index(new_index)
                state.suppressed_events.append(event)
                return False
            remove = _add_hotkey_step(handler, steps[state.index], suppress)
        state.remove_last_step = remove
        state.last_update = _time.monotonic()
        return False
    set_index(0)

    allowed_keys_by_step = [
        set().union(*step)
        for step in steps
    ]

    def remove_():
        state.remove_catch_misses()
        state.remove_last_step()
        del _hotkeys[hotkey]
        del _hotkeys[remove_]
        del _hotkeys[callback]
    # TODO: allow multiple callbacks for each hotkey without overwriting the
    # remover.
    _hotkeys[hotkey] = _hotkeys[remove_] = _hotkeys[callback] = remove_
    return remove_

register_hotkey = add_hotkey
'''

exec(code, keyboard.__dict__)

@singleton
class MonkeyPatch(object):

	def keyboard(self):
		mouzz_log("MonkeyPatch.keyboard")
		f = keyboard.key_to_scan_codes
		mapping = {}
		cache = {}
		def key_to_scan_codes(key, error_if_missing=True):
			if key in cache:
				sv = f(key, error_if_missing)
				if sv != cache[key]:
					log("key_to_scan_codes key %s error_if_missing %s sv %s cache[key] %s", key, error_if_missing, svn, cache[key])
					os.exit(1)
				return cache[key]
			v = f(key, error_if_missing)
			if type(v) is tuple:
				for e in v:
					if e not in mapping:
						mapping[e] = key
			else:
				mapping[v] = key
			cache[key] = v
			log("key_to_scan_codes key %s error_if_missing %s v %s", key, error_if_missing, v)
			return v
		keyboard.key_to_scan_codes = key_to_scan_codes

		# NOTE 这个函数不存在 只是用来 debug
		def scan_codes_to_key(v):
			if type(v) is tuple:
				return tuple([mapping.get(k, k) for k in v])
			return mapping.get(v, v)
		keyboard.scan_codes_to_key = scan_codes_to_key
		
		# keyboard 模块不知道为什么有时候会有 vk == 7 scan_code == 0 的 down 事件 却没有的 up 事件
		# return callback(KeyboardEvent(event_type=event_type, scan_code=scan_code or -vk, name=name, is_keypad=is_keypad))
		# 导致 _pressed_events 会残留一个 -7，所有的 hotkey 都会多了一个 -7 的组合，影响到 hotkey 的判断

		_direct_callback = keyboard._KeyboardListener.__dict__["direct_callback"]
		def direct_callback(self, event):
			ret = _direct_callback(self, event)
			shit = False
			for k, v in keyboard._pressed_events.items():
				if k < 0:
					del keyboard._pressed_events[k]
					log("!" * 79)
					log("delete invalid _pressed_events", k)
					log("!" * 79)
					shit = True
			if shit:
				os.exit(1)
			return ret
		setattr(keyboard._KeyboardListener, "direct_callback", direct_callback)




# encoding: utf8

import os
import json
import time
import ctypes
import atexit
import logging
import argparse
import functools

import mouse
import keyboard
import patch_keyboard

from utils import *
from functools import partial

args = None
my_dir = os.path.dirname(os.path.abspath(__file__))

@singleton
class G(object):

	Mode = None

builtins["G"] = G

@singleton
class Config(object):

	def init(self):
		path = os.path.realpath(args.config)
		if os.path.isfile(path):
			with open(path, "r+") as rf:
				self.data = json.load(rf)
		else:
			import config
			self.data = json.loads(config.config)
			print "use default config"

	def get(self, key, defv=None):
		return self.data.get(key, defv)

@singleton
class Keys(object):

	def init(self):
		keys = Config.get("keys")
		self.left = keys["mouse_move_left"]
		self.right = keys["mouse_move_right"]
		self.up = keys["mouse_move_up"]
		self.down = keys["mouse_move_down"]

		for name in [
				"chrome", "move_window", "direction", 
				"mouse_left_key", "mouse_right_key", "mouse_wheel_up", "mouse_wheel_down",
				"page_up", "page_down", "goto_top", "goto_bottom",
				"kill", "copy", "paste",
				"keep_press_mouse_left_key", "grab_screen",
				"enter_insert_mode", "enter_normal_mode",
				]:
			setattr(self, name, keys.get(name))

@singleton
class MoveMouse(object):

	""" 模拟鼠标移动 """

	def __init__(self):
		self.is_move_left = False
		self.is_move_right = False
		self.is_move_up = False
		self.is_move_down = False

	def tick(self):
		if G.Mode is not NormalMode:
			return
		if keyboard.is_pressed('alt+tab'):
			return
		# TODO win+tab?

		dx = dy = 0

		c = Config.get("mouse_speed")
		dd = c[""]
		for key, r in c.items():
			# 不能使用 alt 作为速度档位，因为 alt + tab 是 windows 默认的切换窗口的快捷键。
			if "alt" in key:
				raise Exception("cannot use alt key as speed key: %s" % key)
			if key and keyboard.is_pressed(key):
				dd = r
				break

		if self.is_move_left:
			dx = -1 * dd
		if self.is_move_right:
			dx = dd
		if self.is_move_up:
			dy = -1 * dd 
		if self.is_move_down:
			dy = dd

		if dx or dy: 
			mouse.move(dx, dy, absolute=False)

@singleton
class Common(object):

	""" 一些需要在多个 Mode 下都能工作(可能功能不同的)的键位 """

	def __init__(self):
		self.hotkey_list = []
		self.switching_window = False

	def setup_direction_keys(self):
		""" hjkl 在 NorlmalMode 下表示移动，或在切换窗口时表示方向键 """

		def on_key(k, v, key_event):
			log("Common.setup_direction_keys is_pressed", k, v, key_event, keyboard.is_pressed('alt'))
			is_press = key_event.event_type == keyboard.KEY_DOWN

			if G.Mode is NormalMode:
				# chrome left tab or right tab
				if k == Keys.left or k == Keys.right:
					if keyboard.is_pressed(Keys.chrome):
						log("chrome tab swtich")
						v = "page down" if k == keys.right else "page up"
						if is_press:
							keyboard.press("ctrl+%s" % v)
						else:
							keyboard.release("ctrl+%s" % v)
						return

				# 窗口停靠
				if keyboard.is_pressed(Keys.move_window):
					if is_press:
						log("windows_alt_tab_remap press windows", v)
						keyboard.press("windows+%s" % v)
					else:
						log("windows_alt_tab_remap release windows",v)
						keyboard.release("windows+%s" % v)
					return 

				# 方向键
				if keyboard.is_pressed(Keys.direction):
					if is_press:
						log("windows_alt_tab_remap press forward", v)
						keyboard.press(v)
					else:
						log("windows_alt_tab_remap release forward",v)
						keyboard.release(v)
					return 

			if self.switching_window:
				key = v # 替换成上下左右
				log("switching_window")
			else:
				key = k # 保持不变

			# 正在切换窗口或者在输入模式的话，只是替换键位
			if key == v or G.Mode is InsertMode:
				if is_press:
					log("windows_alt_tab_remap press down", key)
					keyboard.press(key)
				else:
					log("windows_alt_tab_remap release up", key)
					keyboard.release(key)
					if self.switching_window:
						keyboard.call_later(move_mouse_to_actived_window, delay=0.5)
			# 此时模拟鼠标移动 NormalMode 且不在切换窗口，直接吃掉键盘消息
			else :
				for name in ["up", "down", "left", "right"]:
					if key == getattr(Keys, name):
						setattr(MoveMouse, "is_move_%s" % name, is_press)

		for name in ["up", "down", "left", "right"]:
			k = getattr(Keys, name)
			v = name
			callback = functools.partial(on_key, k, v)
			keyboard.hook_key(k, callback, suppress = True)
			# callback = functools.partial(on_key, False, k, v)
			# keyboard.on_release_key(k, callback, suppress = True)
			log("Common.setup_direction_keys k %s v %s", k, v)

	def listen_alt_tab(self):
		""" 监控是否正在使用 alt+tab 切换窗口 """
		def e(event):
			if keyboard.is_pressed('alt') and keyboard.is_pressed('tab'):
				self.switching_window = True
			log("switch_window enter", self.switching_window)

		def l(event):
			self.switching_window = False
			log("switch_window leave", self.switching_window)

		keyboard.on_press_key("tab", e)
		keyboard.on_press_key("alt", e)
		keyboard.on_release_key("alt", l)

	def backup(self):
		# 使用 w 作为控制键，会影响到正常输入，比如说说 down 会不好输入
		"""
		def null(key):
			log("left")
			keyboard.send('delete, left')
		key = 'w+n'
		self.append_hotkey(key, keyboard.add_hotkey(key, partial(null, key), suppress=True))
		def null(key):
			log("right")
			keyboard.send('delete, right')
		key = 'w+m'
		self.append_hotkey(key, keyboard.add_hotkey(key, partial(null, key), suppress=True))
		def null(key):
			log("mouse right")
			keyboard.send('delete')
			time.sleep(0.01)
			mouse.click(mouse.RIGHT)
		key = 'w+p'
		self.append_hotkey(key, keyboard.add_hotkey(key, partial(null, key), suppress=True))
		"""

class Mode(object):

	def __init__(self):
		self.cleanup_list = []
		self.hotkey_list = []

	def enter(self):
		log("%s.enter" % self.__class__.__name__)
		self.on_enter()

	def on_enter(self):
		pass

	def leave(self):
		log("%s.leave" % self.__class__.__name__)

		self.on_leave()

		for name, c in self.cleanup_list:
			log("%s unhook %s %s" % (self.__class__.__name__, c, name))
			keyboard.unhook_key(c)
		for c in self.hotkey_list:
			log("%s remove hotkey %s %s" % (self.__class__.__name__, c, getattr(c, "key", "...")))
			keyboard.remove_hotkey(c)
		self.cleanup_list = []
		self.hotkey_list = []

	def on_leave(self):
		pass

	def append_cleanup(self, name, func):
		self.cleanup_list.append((name, func))

	def append_hotkey(self, key, func):
		func.key = key
		self.hotkey_list.append(func)

@singleton
class InsertMode(Mode):

	def on_enter(self):
		self.setup_change_mode()
		switch_chinese_input_method()
		print_current_state(repeat=False)

	def setup_change_mode(self):
		def enter_normal_mode(key):
			log("enter_normal_mode %s %s", G.Mode, key)
			if G.Mode is not NormalMode:
				switch_mode(NormalMode)
			else:
				keyboard.press(key)
		key = Keys.enter_normal_mode
		self.append_hotkey(key, keyboard.add_hotkey(key, functools.partial(enter_normal_mode, key), suppress=True))

@singleton
class NormalMode(Mode):

	def __init__(self):
		Mode.__init__(self)

		self.grab_screen_pos = [(-1, -1), (-1, -1)]
		self.keep_press_mouse_left_key = False

	def on_enter(self):
		self.setup_change_mode()
		self.setup_mouse_key()
		self.setup_mouse_move()
		self.setup_mouse_wheel()
		self.setup_page_up_down()
		self.setup_kill_window()
		self.setup_copy_paste()
		self.setup_grab_screen()
		switch_english_input_method()

	def setup_grab_screen(self):

		def grab_screen(key):
			log("grab_screen", key, self.grab_screen_pos)
			if self.grab_screen_pos[0] == (-1, -1):
				self.grab_screen_pos[0] = mouse.get_position()
				log("grab_screen begin", key, self.grab_screen_pos)
			else:
				self.grab_screen_pos[1] = mouse.get_position()
				log("do grab_screen", key, self.grab_screen_pos)
				# log(tuple(self.grab_screen_pos[0] + self.grab_screen_pos[1]))

				a, b = self.grab_screen_pos[0] 
				c, d = self.grab_screen_pos[1] 
				if a > c: 
					a, c = c, a
				if b > d: 
					b, d = d, b
				a *= 2
				b *= 2
				c *= 2
				d *= 2

				log("do grab_screen", key, a, b, c, d)
				from PIL import ImageGrab
				image = ImageGrab.grab(bbox = (a, b, c, d), all_screens=True)
				image.save("png.png")

				# parameter must be a PIL image 
				def send_to_clipboard(image):
					from io import BytesIO
					output = BytesIO()
					image.convert('RGB').save(output, 'BMP')
					data = output.getvalue()[14:]
					output.close()
					
					import win32clipboard
					win32clipboard.OpenClipboard()
					win32clipboard.EmptyClipboard()
					win32clipboard.SetClipboardData(win32clipboard.CF_DIB, data)
					win32clipboard.CloseClipboard()

				send_to_clipboard(image)

				self.grab_screen_pos = [(-1, -1), (-1, -1)]

		key = Keys.grab_screen
		self.append_hotkey(key, keyboard.add_hotkey(key, partial(grab_screen, key), suppress=True))

	def setup_change_mode(self):
		def leave_normal_mode(key):
			log("leave_normal_mode %s %s", G.Mode, key)
			if G.Mode is NormalMode:
				switch_mode(InsertMode)
			else:
				keyboard.press(key)

		def enter_insert_mode(key):
			leave_normal_mode(key)
			log("enter_insert_mode")
			mouse.click()
		key = Keys.enter_insert_mode
		self.append_hotkey(key, keyboard.add_hotkey(key, partial(enter_insert_mode, key), suppress=True))

	def setup_kill_window(self):
		def kill(key):
			log("kill", key)
			mouse.click(mouse.LEFT)
			keyboard.send('alt+f4')
		key = Keys.kill
		self.append_hotkey(key, keyboard.add_hotkey(key, functools.partial(kill, key), suppress=True))

	def setup_copy_paste(self):
		def copy(key):
			log("copy")
			keyboard.send('ctrl+c')
			if self.keep_press_mouse_left_key:
				self.keep_press_mouse_left_key = False
				mouse.release(mouse.LEFT)
		key = Keys.copy
		self.append_hotkey(key, keyboard.add_hotkey(key, partial(copy, key), suppress=True))

		def paste(key):
			log("paste")
			if is_using_putty():
				mouse.click(mouse.RIGHT)
			else:
				keyboard.send('ctrl+v')
		key = Keys.paste
		self.append_hotkey(key, keyboard.add_hotkey(key, partial(paste, key), suppress=True))

	def setup_page_up_down(self):
		def page_up(key):
			exe = get_forceground_window_executable_path()
			log("page up", exe)
			if "chrome" in exe:
				n = 20
				keys = ["up" for i in range(n)]
				keys = ",".join(keys)
				keyboard.send(keys)
			else:
				keyboard.send("page up")
		key = Keys.page_up
		self.append_hotkey(key, keyboard.add_hotkey(key, partial(page_up, key), suppress=True))

		def page_down(key):
			exe = get_forceground_window_executable_path()
			log("page down", exe)
			if "chrome" in exe:
				n = 20
				keys = ["down" for i in range(n)]
				keys = ",".join(keys)
				keyboard.send(keys)
			else:
				keyboard.send("page down")
		key = Keys.page_down
		self.append_hotkey(key, keyboard.add_hotkey(key, partial(page_down, key), suppress=True))

	def setup_mouse_move(self):
		# TODO 不要这几行有没有问题
		key = 'd'
		self.append_hotkey(key, keyboard.add_hotkey(key, partial(null, key), suppress=True))
		key = 'r'
		self.append_hotkey(key, keyboard.add_hotkey(key, partial(null, key), suppress=True))
		key = 'f'
		self.append_hotkey(key, keyboard.add_hotkey(key, partial(null, key), suppress=True))

		def on_mouse_move_d(key, dx, dy):
			log("on_mouse_move_d", key, dx, dy)
			mouse.move(dx, dy, absolute=False)
		for key, (dx, dy) in Config.get("mouse_jump").items():
			self.append_hotkey(key, keyboard.add_hotkey(key, partial(on_mouse_move_d, key, dx, dy), suppress=True))

		def goto_top_or_bottom(key, top):
			log("goto_top_or_bottom", key, top)
			cx, _ = mouse.get_position()
			if top:
				mouse.move(cx, 0, absolute=True)
			else:
				y = max_screen_y()
				log("goto_top_or_bottom", key, top, y)
				mouse.move(cx, y, absolute=True)

		for key, is_top in [[Keys.goto_top, True], [Keys.goto_bottom, False]]:
			self.append_hotkey(key, keyboard.add_hotkey(key, partial(goto_top_or_bottom, key, is_top), suppress=True))

	def setup_mouse_wheel(self):
		def mouse_wheel(key, delta):
			log("mouse wheel", key)
			mouse.wheel(delta)
		for key, delta in [[Keys.mouse_wheel_up, 1], [Keys.mouse_wheel_down, -1]]:
			self.append_hotkey(key, keyboard.add_hotkey(key, partial(mouse_wheel, key, delta), suppress=True))

	def setup_mouse_key(self):
		def on_press_mouse_key(key, mouse_key, key_event):
			log("mouse_key_setup press", key, mouse_key, key_event, self.keep_press_mouse_left_key)
			if self.keep_press_mouse_left_key:
				return
			mouse.press(mouse_key)

		def on_release_mouse_key(key, mouse_key, key_event):
			log("mouse_key_setup release", key, mouse_key, key_event, self.keep_press_mouse_left_key)
			if self.keep_press_mouse_left_key:
				return
			mouse.release(mouse_key)

		def on_mouse_key(key, mouse_key, key_event):
			log("mouse_key_setup on_mouse_key", key, mouse_key, key_event, self.keep_press_mouse_left_key)
			if self.keep_press_mouse_left_key:
				return
			if key_event.event_type == keyboard.KEY_DOWN:
				log("mouse press", mouse_key)
				mouse.press(mouse_key)
			if key_event.event_type == keyboard.KEY_UP:
				log("mouse release", mouse_key)
				mouse.release(mouse_key)

		for key, mouse_key in [[Keys.mouse_left_key, mouse.LEFT], [Keys.mouse_right_key, mouse.RIGHT]]:
			self.append_cleanup("hook_key %s" % key, keyboard.hook_key(key, partial(on_mouse_key, key, mouse_key), suppress=True))
			# self.append_cleanup("press %s" % key, keyboard.hook_key(key, partial(on_mouse_key, key, mouse_key)))

			# NOTE 这样写 keyboard 会挂 File "C:\Python27\lib\site-packages\keyboard\__init__.py", line 501, in remove_ del _hooks[key]
			# self.append_cleanup("press %s" % key, keyboard.on_press_key(key, partial(on_press_mouse_key, key, mouse_key), suppress=True))
			# self.append_cleanup("release %s" % key, keyboard.on_release_key(key, partial(on_release_mouse_key, key, mouse_key), suppress=True))

		def keep_press_mouse_left_key(key):
			log("keep_press_mouse_left_key", key, self.grab_screen_pos)
			if self.keep_press_mouse_left_key:
				self.keep_press_mouse_left_key = False
				mouse.release(mouse.LEFT)
			else:
				self.keep_press_mouse_left_key = True
				mouse.press(mouse.LEFT)

		key = Keys.keep_press_mouse_left_key
		self.append_hotkey(key, keyboard.add_hotkey(key, partial(keep_press_mouse_left_key, key), suppress=True))

def switch_mode(new_mode):
	if G.Mode is new_mode:
		return
	log("swtich_mode leave %s", G.Mode)
	G.Mode and G.Mode.leave()
	G.Mode = new_mode
	log("swtich_mode enter %s", G.Mode)
	G.Mode.enter()

def init():
	logging.basicConfig(filename = "./mouzz.log", filemode = "a", level = logging.DEBUG, format = '%(asctime)s - %(levelname)s: %(message)s', datefmt = '%m/%d/%Y %I:%M:%S %p')
	patch_keyboard.MonkeyPatch.keyboard()
	atexit.register(cleanup_all_keyboard_hook)

	parser = argparse.ArgumentParser()
	parser.add_argument("config", type=str, default="", help="config file path", nargs='?')

	global args
	args, unknow = parser.parse_known_args()

	Config.init()
	Keys.init()

def main():
	init()

	Common.setup_direction_keys()
	Common.listen_alt_tab()

	switch_mode(InsertMode)

	keyboard.call_later(print_current_state, delay=2)

	frames_per_second = Config.get("mouse_move_frames")
	while True:
		time.sleep(1.0 / frames_per_second)
		MoveMouse.tick()

if __name__ == "__main__":
	main()



# encoding: utf8

import os
import time
import types
import ctypes
import atexit
import logging
import functools
import threading

import mouse
import keyboard

from functools import partial

# buildin
#=============================================================================
builtins = globals()["__builtins__"]
if type(builtins) is types.ModuleType:
	builtins = builtins.__dict__

# log
#=============================================================================

lock = threading.Lock()

def tostr(s):
	if type(s) is unicode:
		return s.encode('utf8')
	return s

def mouzz_log(fmt, *args):
	if args:
		try:
			fmt = fmt % args
		except TypeError:
			fmt = fmt + " " + " ".join([str(tostr(arg)) for arg in args])

	fmt = time.strftime("%Y-%m-%d %H:%M:%S") + " " + str(threading.current_thread().ident) + " " + fmt
	with lock:
		print(fmt)
		logging.info(fmt)

log = mouzz_log

# for debug keyboard module
builtins["mouzz_log"] = log 

# python
#=============================================================================

def singleton(klass):
	return klass()

def print_object_dir(obj):
	for attr in dir(obj):
		log("  ", attr, getattr(obj, attr, "..."))

# screen
#=============================================================================

def max_screen_y():
	user32 = ctypes.windll.user32
	# screensize = user32.GetSystemMetrics(0), 
	y = user32.GetSystemMetrics(1)
	return y

# state
#=============================================================================

def print_current_state(repeat=True):
	return
	log("-" * 79)
	if repeat: 
		keyboard.call_later(print_current_stat, delay=2)

	if G.Mode is NormalMode:
		log("In NormalMode", mouse.get_position())
	else:
		log("In InsertMode", mouse.get_position())

	l = []
	for key in keyboard._hooks.keys():
		if type(key) is str:
			l.append(key)
	log("hook key", " ".join(l))
	l = []
	for key in keyboard._hotkeys.keys():
		if type(key) is str:
			l.append(key)
	log("hot key", " ".join(l))

	l = []
	for key in keyboard.all_modifiers:
		if keyboard.is_pressed(key):
			l.append(key)
	log("modifer key", " ".join(l))

	l = []
	for k, v in keyboard._listener.blocking_hotkeys.items():
		if v:
			l.append((keyboard.scan_codes_to_key(k), k))
	log("hot keys cb", l)
	# log("get_forceground_window", get_forceground_window())

	user32 = ctypes.WinDLL('user32', use_last_error=True)
	curr_window = user32.GetForegroundWindow()
	thread_id = user32.GetWindowThreadProcessId(curr_window, 0)
	klid = user32.GetKeyboardLayout(thread_id)
	lid = klid & (2**16 - 1)
	lid_hex = hex(lid)
	 
	print(klid, lid, lid_hex)
	print(klid, lid, lid_hex)
	print(klid, lid, lid_hex)
	# if lid_hex == '0x409':
	#     print(u'当前的输入法状态是英文输入模式\n\n')
	# elif lid_hex == '0x804':
	#     print(u'当前的输入法是中文输入模式\n\n')
	# else:
	#     print(u'当前的输入法既不是英文输入也不是中文输入\n\n')

	# global IME
	# if IME == 0x0409:
	# 	switch_input_method(0x0804)
	# else:
	# 	switch_input_method(0x0409)
	log("-" * 79)

# windows
#=============================================================================

def get_forceground_window():
	from win32gui import GetWindowText, GetForegroundWindow
	return GetWindowText(GetForegroundWindow())

def get_forceground_window_executable_path():
	from win32gui import GetForegroundWindow
	import win32process, psutil
	pid = win32process.GetWindowThreadProcessId(GetForegroundWindow())
	return psutil.Process(pid[-1]).exe()

def is_using_putty():
	return "putty" in get_forceground_window_executable_path().lower()

def is_using_chrome():
	return "chrome" in get_forceground_window_executable_path().lower()

def move_mouse_to_actived_window():
	from win32gui import GetWindowText, GetForegroundWindow, GetWindowRect
	hwnd = GetForegroundWindow()
	text = GetWindowText(hwnd)
	if type(text) is str: 
		try:
			text = text.decode("utf8")
		except Exception, e:
			try:
				text = text.decode("gbk")
			except Exception, e:
				pass
	if text == u"任务切换":
		return
	rect = GetWindowRect(hwnd)
	x0, y0, x1, y1 = rect
	log("-" * 79)
	log(text)
	log("move_mouse_to_actived_window", text, rect)
	mouse.move((x0 + x1) / 2, (y0 + y1) / 2, absolute=True)
	# mouse.move(x0 + 100, y0 + 100, absolute=True)

# mouse
#=============================================================================

def change_mouse_cursor():
	return
	import win32gui, ctypes, win32con
	hold = win32gui.LoadImage(0, 32512, win32con.IMAGE_CURSOR, 0, 0, win32con.LR_SHARED )
	hsave = ctypes.windll.user32.CopyImage(hold, win32con.IMAGE_CURSOR, 0, 0, win32con.LR_COPYFROMRESOURCE)
	path = r"C:\Windows\Cursors\move_m.cur"
	hnew = win32gui.LoadImage(0, path, win32con.IMAGE_CURSOR, 0, 0, win32con.LR_LOADFROMFILE);
	ctypes.windll.user32.SetSystemCursor(hnew, 32512)
	time.sleep(10)
	# restore the old cursor
	ctypes.windll.user32.SetSystemCursor(hsave, 32512)

# keyboard
#=============================================================================

"""
- on_press / on_release 面向任何键
  - hook
  - unhook
  - blocking_hooks
	- suppress
	  - 在 _KeyboardListener.direct_callback() 中所有的 event 会先被 blocking_hooks 处理
	  - 可以中途返回
  - add_handler / remove_handler
	- 非 suppress
	  - 在 _KeyboardListener.direct_callback() 中所有的 event 无条件放到 self.queue
	  - 进而被 handlers 在另外一个线程单独处理

- on_press_key / on_release_key / block_key 面向单个键
  - hook_key
  - unkook_key / unblock_key
  - blocking_keys
	- suppress
	  - 在 _KeyboardListener.direct_callback() 中所有的 event 在处理 blocking_hooks 之后会处理 blocking_keys
	  - 可以中途返回
  - nonblocking_keys
	- 非 suppress
	  - 在 _KeyboardListener.direct_callback() 中所有的 event 无条件放到 self.queue
	  - 进而被 pre_process_event 在另外一个线程单独处理

- add_hotkey hotkey 不能区分按下按起的两种事件
  - remove_hotkey
  - remove_all_hotkey
  - remap_hotkey
  - blocking_hotkeys
	- suppress
	  - 在 _KeyboardListener.direct_callback() 中所有的 event 在处理 blocking_keys 之后会处理 blocking_hotkeys
  - nonblocking_hotkeys
	- 非 suppress
	  - 在 _KeyboardListener.direct_callback() 中所有的 event 无条件放到 self.queue
	  - 进而被 pre_process_event 在另外一个线程单独处理


"""

def null(key):
	log("null", key)

def cleanup_all_keyboard_hook(*args, **kw):
	log("cleanup_all_keyboard_hook")
	keyboard.unhook_all()

# input method
#=============================================================================

IME = 0x0409

def switch_chinese_input_method():
	switch_input_method(ime=0x0804)

def switch_english_input_method():
	switch_input_method(ime=0x0409)

def switch_input_method(ime=0x0409):
	from win32con import WM_INPUTLANGCHANGEREQUEST
	import win32gui
	import win32api
	
	# 语言代码
	# https://msdn.microsoft.com/en-us/library/cc233982.aspx
	LID = {0x0804: "Chinese (Simplified) (People's Republic of China)",
	       0x0409: 'English (United States)'}
	
	# 获取前景窗口句柄
	hwnd = win32gui.GetForegroundWindow()
	
	# 获取前景窗口标题
	title = win32gui.GetWindowText(hwnd)
	# print(u'当前窗口：' + title)
	
	# 获取键盘布局列表
	im_list = win32api.GetKeyboardLayoutList()
	im_list = list(map(hex, im_list))
	
	# 设置键盘布局为英文
	result = win32api.SendMessage(
	    hwnd,
	    WM_INPUTLANGCHANGEREQUEST,
	    0,
	    ime)
	if result == 0:
		global IME
		IME = ime
	log('switch_input_method', im_list, ime, result)




config = """{
	"keys": {
		"keep_press_mouse_left_key": "v",
		"grab_screen": "c",

		"kill": "q, q",
		"copy": "y",
		"paste": "p",

		"page_up": "ctrl+b",
		"page_down": "ctrl+f",
		"goto_top": "g, g",
		"goto_bottom": "G",

		"mouse_left_key": "space",
		"mouse_right_key": "n",
		"mouse_wheel_up": ",",
		"mouse_wheel_down": "m",

		"enter_insert_mode": "i",
		"enter_normal_mode": "ctrl+`",

		"mouse_move_down": "j",
		"mouse_move_up": "k",
		"mouse_move_left": "h",
		"mouse_move_right": "l",

		"move_window": "w",
		"direction": "r",
		"chrome": "c"
	}, 

	"mouse_move_frames": 200,
	"mouse_speed": {
		"": 10,
		"d": 30,
		"f": 2
	}, 

	"mouse_jump": {
		"b": [-400, 0],
		"e": [400, 0]
	}

}
"""




#!/usr/bin/env python
# coding: utf-8

from setuptools import setup

setup(
    name='mouzz',
    version='0.0.10',
    author='prim',
    author_email='1210102@qq.com',
    url='https://github.com/prim',
    description=u'mouzz',
    packages=['mouzz'],
    install_requires=["mouse", "keyboard", "psutil", "pywin32"],
    entry_points={
        'console_scripts': [
            'mouzz=mouzz:main',
        ]
    }
)
